import { createApp } from 'vue'
import './style.css'
import { init } from '@piveau/piveau-hub-ui-plugin-auth'
import { QueryClient, VueQueryPlugin } from '@tanstack/vue-query'
import App from './App.vue'
import router from './router'
import { useEnv } from './useEnv'

const queryClient = new QueryClient()

const env = useEnv()

const { plugin, navigationGuardFn } = init({
  baseUrl: env.VITE_APP_PV_AUTH_MIDDLEWARE_API_URL || '/api',
  refreshInterval: 2 * 60 * 1000,
  defaultLoginOptions: {
    redirect_url: env.VITE_APP_PV_AUTH_REDIRECT_URL || '/login-redirect',
  },
  queryClient,
  enabled: true,
})

router.beforeEach(navigationGuardFn)

const app = createApp(App)
app.use(VueQueryPlugin, {
  queryClient,
})
app.use(plugin)
app.use(router)

app.mount('#app')
