/// <reference types="vitest/config" />

import { fileURLToPath } from 'node:url'
import { defineConfig } from 'vite'
import AutoImport from 'unplugin-auto-import/vite'
import Components from 'unplugin-vue-components/vite'
import Icons from 'unplugin-icons/vite'
import vue from '@vitejs/plugin-vue'

// https://vitejs.dev/config/
export default defineConfig({
  plugins: [AutoImport(), Components(), Icons(), vue()],

  server: {
    port: 3000,

    // proxy requests to https://piveau-auth-middleware-evaluation.apps.osc.fokus.fraunhofer.de/ as /api due to CORS issues
    proxy: {
      '/api': {
        target:
          'https://piveau-auth-middleware-evaluation.apps.osc.fokus.fraunhofer.de/',
        changeOrigin: true,
        rewrite: path => path.replace(/^\/api/, ''),
      },
    },
  },

  resolve: {
    alias: {
      '@piveau/piveau-hub-ui-plugin-auth': fileURLToPath(
        new URL('../../src/index.ts', import.meta.url),
      ),
    },
  },

  test: {
    environment: 'happy-dom',
  },
})
