import type { OauthRequestParameters } from '../types'
import type { createPiveauAuthApi } from './api'

export interface NavigateToLoginPageOptions {
  api: ReturnType<typeof createPiveauAuthApi>
  redirectUrl?: string
  requestParameters?: OauthRequestParameters
}

export async function navigateToLoginPage({
  api,
  redirectUrl,
  requestParameters,
}: NavigateToLoginPageOptions) {
  const { authorization_request_url: authorizationRequestUrl } = await api.loginStart({
    requestParameters: {
      ui_locales: 'en',
      acr_values: 'https://ecas.ec.europa.eu/loa/basic',
      prompt: 'login',
      display: 'page',
      ...requestParameters,
    },
    page_url: window.location.href,
    redirect_url: redirectUrl || window.location.href,
  })

  window.location.href = authorizationRequestUrl
}
