import { type ComputedRef, type MaybeRef, unref } from 'vue-demi'
import type { QueryClient } from '@tanstack/vue-query'
import type { PiveauAuthApi } from './api'

export function createNavigationGuard({
  enabled = true,
  api,
  onRequiredAuth,
  isLoggedIn,
  qc,
  metaPropertyName = 'requiresAuth',
}: {
  enabled?: MaybeRef<boolean>
  api: PiveauAuthApi
  onRequiredAuth: (api: PiveauAuthApi) => void
  isLoggedIn: ComputedRef<boolean>
  qc: QueryClient
  metaPropertyName?: string
}) {
  return async (to: any, _from: any, next: any) => {
    if (
      !isLoggedIn.value
      && to.matched.some((record: any) => record.meta[metaPropertyName])
    ) {
      const isEnabled = unref(enabled)
      if (!isEnabled) {
        next(false)
        return
      }
      try {
        const {
          is_logged_in: isLoggedIn,
          stored_page_url: storedUrl,
        }
        = await qc.fetchQuery({
          queryKey: ['loginEnd'],
          queryFn: () => api.loginEnd({ page_url: window.location.href }),
          staleTime: Number.POSITIVE_INFINITY,
          gcTime: Number.POSITIVE_INFINITY,
        })
        if (isLoggedIn) {
          const maybeStoredUrlPath = storedUrl
            ? new URL(storedUrl).pathname
            : undefined

          next(maybeStoredUrlPath)
          return
        }

        onRequiredAuth(api)
        return
      }
      catch (error) {
        console.error('Error during loginEnd:', error)
        next(false)
        return
      }
    }
    next()
  }
}
