import { defineComponent, getCurrentInstance, h, inject, onBeforeMount, isVue2 } from 'vue-demi'
import { key, useLoginRedirectHandler } from '..'

export default defineComponent({
  name: 'LoginRedirect',

  displayName: 'LoginRedirect',

  props: {
    as: {
      type: String,
      default: 'div',
    },
  },

  setup(props, { slots, attrs }) {
    const pvAuth = inject(key)

    const vm = getCurrentInstance()
    if (!vm)
      throw new Error('This function must be called within the setup function.')

    const router = isVue2
      ? (vm?.proxy as any)?.$router
      : vm?.appContext.config.globalProperties.$router
      
    const routerBase =
      (router?.options?.history?.base as string)
      || (router?.options?.memory?.base as string)
      || (router?.options?.base as string)
      || '/'

    const { start } = useLoginRedirectHandler({
      redirectUrl: window.location.href,
    })

    function extractPathFromUrl(url: string): string {
      try {
        const parsedUrl = new URL(url)
        return parsedUrl.pathname + parsedUrl.search
      }
      catch (e) {
        console.error('Invalid URL provided:', e)
        return ''
      }
    }

    onBeforeMount(async () => {
      if (!pvAuth)
        return

      start((url) => {
        const fullpath = extractPathFromUrl(url)

        const basePathAwarePath = fullpath.startsWith(routerBase)
          ? fullpath.slice(routerBase.length)
          : fullpath

        router.push(basePathAwarePath)
      })
    })

    return () => h(props.as, attrs, slots)
  },
})
