import type { App } from '@vue/devtools-api'
import { setupDevtoolsPlugin } from '@vue/devtools-api'
import type { PiveauAuth } from './types'

const color = {
  successBg: 0x16A34A,
  successFg: 0xF7FEE7,
  warningBg: 0xD97706,
  warningFg: 0xFFFBEB,
} as const

export function setupDevtools(app: App, data: PiveauAuth) {
  const stateType = 'pvAuth' as const
  setupDevtoolsPlugin({
    id: 'piveau-hub-ui-plugin-auth',
    label: 'Piveau Hub UI Plugin Auth',
    componentStateTypes: [stateType],
    app,
  }, (api) => {
    api.on.inspectComponent((payload, _context) => {
      if (payload.instanceData) {
        payload.instanceData.state.push({
          type: stateType,
          key: '$pvAuth',
          editable: false,
          value: data,
        })
      }
    })

    api.addInspector({
      id: 'pvAuth',
      label: 'Piveau Auth',
      icon: 'lock',
    })

    api.on.getInspectorTree((payload, _context) => {
      const isAuthenticated = data.authData.value.isAuthenticated
      const username = isAuthenticated && data.authData.value.username

      if (payload.app === app && payload.inspectorId === 'pvAuth') {
        payload.rootNodes = [
          {
            id: 'auth',
            label: 'Auth',
            tags: [
              {
                label: isAuthenticated ? username || 'Authenticated' : 'Not authenticated',
                textColor: isAuthenticated ? color.successFg : color.warningFg,
                backgroundColor: isAuthenticated ? color.successBg : color.warningBg,
              },
            ],
            children: [
              {
                id: 'options',
                label: 'Plugin Options',
              },
              {
                id: 'authflow',
                label: 'Authentication Flow',
              },
              {
                id: 'authdata',
                label: 'authData',
              },
              // {
              //   id: 'tokenmanagement',
              //   label: 'Token Management',
              // },
            ],
          },
        ]
      }
    })

    api.on.getInspectorState((payload, _context) => {
      if (payload.inspectorId === 'pvAuth') {
        if (payload.nodeId === 'options') {
          payload.state = {
            Options: [
              {
                key: 'enabled',
                value: data.options.enabled,
              },
              {
                key: 'baseUrl',
                value: data.options.baseUrl,
              },
              {
                key: 'refreshInterval',
                value: data.options.refreshInterval,
              },
              {
                key: 'defaultLoginOptions',
                value: data.options.defaultLoginOptions,
              },
              {
                key: 'defaultLogoutOptions',
                value: data.options.defaultLogoutOptions,
              },
            ],
          }
        }
        else if (payload.nodeId === 'authflow') {
          payload.state = {
            redirectionParams: [
              {
                key: 'iss',
                value: data.redirectParams.iss,
              },
              {
                key: 'code',
                value: data.redirectParams.code,
              },
              {
                key: 'state',
                value: data.redirectParams.state,
              },
            ],
          }
        }
        else if (payload.nodeId === 'authdata') {
          payload.state = {
            authData: [
              {
                key: 'isAuthenticated',
                value: data.authData.value.isAuthenticated,
              },
              {
                key: 'username',
                value: data.authData.value.username,
              },
              {
                key: 'idTokenParsed',
                value: data.authData.value.idTokenParsed,
              },
              {
                key: 'resources',
                value: data.authData.value.resources,
              },
              {
                key: 'rpt',
                value: data.authData.value.rpt,
              },
            ],
          }
        }
      }
    })
  })
}
