import type { QueryClient } from '@tanstack/vue-query'
import { useQuery } from '@tanstack/vue-query'
import type { MaybeRef } from 'vue-demi'
import { computed, unref } from 'vue-demi'
import type { PiveauAuthApi } from '../auth/api'
import { useApiAndClient } from '../utils'

export default function useLoginEndQuery(options?: {
  enabled?: MaybeRef<boolean>
  api?: PiveauAuthApi
  qc?: QueryClient
  pageUrl?: string
  csrf?: MaybeRef<string>
}) {
  const { api, qc } = useApiAndClient(options)
  const key = computed(() => ['loginEnd', options?.csrf || ''])
  const query = useQuery({
    queryKey: key,
    enabled: computed(() => unref(options?.enabled)),
    queryFn: ({ queryKey }) => api.loginEnd({ page_url: options?.pageUrl || window.location.href }, {
      headers: {
        ...(queryKey[1] as string
          ? { 'X-PIVEAU-CSRF-PROTECTION': queryKey[1] as string }
          : {}),
      },
    }),
    refetchOnWindowFocus: false,
    staleTime: Number.POSITIVE_INFINITY,
  }, qc)
  return query
}
