import type { QueryClient } from '@tanstack/vue-query'
import { useQuery } from '@tanstack/vue-query'
import type { MaybeRefOrGetter } from 'vue-demi'
import { computed, toValue } from 'vue-demi'
import type { PiveauAuthApi } from '../auth/api'
import { authData } from '../auth/manager'
import { useApiAndClient } from '../utils'

export function useRptQuery(options?: {
  api?: PiveauAuthApi
  queryKey?: MaybeRefOrGetter<string[]>
  enabled?: MaybeRefOrGetter<boolean>
  qc?: QueryClient
}) {
  const { api, qc } = useApiAndClient(options)

  const key = computed(() => ['rpt', ...(toValue(options?.queryKey) || [])])

  const query = useQuery({
    queryKey: key,
    enabled: computed(() => authData.isAuthenticated && !!toValue(options?.enabled)),
    queryFn: () => api.token(),
    refetchOnWindowFocus: false,
  }, qc)

  return query
}
