import type { QueryClient } from '@tanstack/vue-query'
import { useQuery } from '@tanstack/vue-query'
import type { MaybeRefOrGetter } from 'vue-demi'
import { computed, toValue } from 'vue-demi'
import type { PiveauAuthApi } from '../auth/api'
import { authData } from '../auth/manager'
import { useApiAndClient } from '../utils'

interface UseUserinfoQueryOptions {
  api?: PiveauAuthApi
  queryKey?: MaybeRefOrGetter<string[]>
  enabled?: MaybeRefOrGetter<boolean>
  qc?: QueryClient
}

/**
 * Composable to fetch and manage user information.
 *
 * @param {UseUserinfoQueryOptions} options - Options to customize the query.
 * @return  The query object containing the user information.
 */
export function useUserinfoQuery(options: UseUserinfoQueryOptions = {}) {
  const { api, qc } = useApiAndClient(options)

  const key = computed(() => ['userinfo', ...toValue(options?.queryKey) || []])

  const query = useQuery({
    queryKey: key,
    enabled: computed(() => authData.isAuthenticated && toValue(options?.enabled)),
    queryFn: () => api.userinfo(),
    refetchOnWindowFocus: false,
  }, qc)

  return query
}
